<?php
namespace App\Controllers;

use App\Models\ItemModel;
use App\Models\OrderModel;
use App\Models\OrderItemModel;
use App\Models\WorkflowModel;
use App\Models\TaskModel;
use App\Models\BatchModel;
use App\Models\BatchItemModel;
use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;
use App\Libraries\Barcode;
use App\Models\AdminRoleModuleModel;
use App\Models\TreeModel;
use App\Models\PrintHistoryModel;
use App\Models\BatchItemTaskModel;
use App\Models\BatchItemRequestModel;
use App\Models\BatchItemOutputModel;
use App\Models\BatchItemGroupNoteModel;
use App\Models\BatchItemTaskLogModel;
use App\Models\OutgoingNoteModel;
use App\Models\OutgoingNoteDetailModel;
use App\Models\OperatorModel;
use App\Models\OperatorTaskModel;

class BatchItemTask extends BaseController
{
	public function index()
	{
		if($this->request->isAjax()){
			return $this->data();
		}

		$data = [
			'title' => lang('Menu.order.batch-item-task.main'),
			'menu' => 'batch-item-task',
		];

		return view('batch-item-task/index', $data);
	}

	public function store($id){

		$batchModel = new BatchModel();

		$batchItemTaskModel = new BatchItemTaskModel();

		$batchData = $batchModel->getByID($id);

		$batchItemModel = new BatchItemModel();
		$workflowModel = new WorkflowModel();

		$batchItemData = $batchItemModel->getByBatchID($id);
		$taskModel = new TaskModel();

		$batchItemRequestModel = new BatchItemRequestModel();
		
		$data = [];

		foreach ($batchItemData as $key => $value) {
			$s3Library = new S3Library();
			$batchItemRequestGet = $batchItemRequestModel->getAll(['where' => 'AND bir_status="New" AND bir_bi_id="'.$value->id.'" AND bir_active=1 ']);

			if(count($batchItemRequestGet) > 0){
				$msg = [
	                'error' => lang('ErrorMessages.materialNotYetApproved'),
	            ];
	            return json_encode($msg);
			}
		}

		if($batchData->total_weight == 0 || $batchData->total_gold_weight == 0){
			
			$msg = [
                'error' => lang('ErrorMessages.batchIncomplete'),
            ];

            return json_encode($msg);
		}


		// foreach ($batchItemData as $key_ => $value_) {

		// 	if ($value_->erp_barcode == '') {
		// 		$msg = [
	    //             'error' => lang('ErrorMessages.batchErpBarcodeIncomplete'),
	    //         ];
	    //         return json_encode($msg);		
		// 	}
		// }


		$arrayBatchID = [];
		foreach ($batchItemData as $key_ => $value_) {
			$arrayBatchID[] = $value_->id;
		}

		if (empty($arrayBatchID)) {
			$msg = [
                'error' => lang('ErrorMessages.somethingError'),
            ];
            return json_encode($msg);
		}

		$checkBatchItemTask = $batchItemTaskModel->getAll(['where' => ' AND bit_bi_id IN ('.implode(',', $arrayBatchID).') ']);

		if (!empty($checkBatchItemTask)) {
			$msg = [
                'error' => lang('ErrorMessages.somethingError'),
            ];
            return json_encode($msg);
		}


		foreach ($batchItemData as $key => $value) {
			$workflowID = $value->workflow_id;

			$workflowTaskData = $workflowModel->getWorkflowTaskGetByWorkflow($workflowID);

			$groupNote = $this->groupNote();

			foreach ($workflowTaskData as $key1 => $value1) {

				$taskInputData = [];
				$taskResult = $taskModel->taskInputFieldGetByTask($value1->task_id, 'input');
				foreach ($taskResult as $key2 => $value2) {
					$taskInputData[$key2]['field_name'] = $value2->name;
					$taskInputData[$key2]['is_required'] = $value2->is_required;
					$taskInputData[$key2]['datatype'] = $value2->datatype;
					$taskInputData[$key2]['options'] = $value2->options;
					$taskInputData[$key2]['answer'] = '';
				}

				$taskOutputData = [];
				$taskResult3 = $taskModel->taskInputFieldGetByTask($value1->task_id, 'output');
				foreach ($taskResult3 as $key3 => $value3) {
					$taskOutputData[$key3]['field_name'] = $value3->name;
					$taskOutputData[$key3]['is_required'] = $value3->is_required;
					$taskOutputData[$key3]['datatype'] = $value3->datatype;
					$taskOutputData[$key3]['options'] = $value3->options;
					$taskOutputData[$key3]['answer'] = '';    
				}

				$data = [
					'bit_bi_id' => $value->id,
					'bit_order' => ($key1+1),
					'bit_group_note' => $groupNote,
					'bit_workflow_id' => $value1->workflow_id,
					'bit_task_id' => $value1->task_id,
					'bit_input_operator_name' => '',
					'bit_output_operator_name' => '',
					'bit_input_fields' => json_encode($taskInputData),
					'bit_output_fields' => json_encode($taskOutputData),
					'bit_input_quantity' => 0,
					'bit_output_quantity' => 0,
					'bit_input_weight' => 0,
					'bit_output_weight' => 0,
					'bit_status_field' => ($key1 == 0)? 'input' : '',
					'bit_status' => ($key1 == 0)? 'Proses' : 'New',
					'bit_insert_at' => getCurrentTime(),
					'bit_insert_admin_id' => $this->session->get('id')
				];

				$result = $batchItemTaskModel->addBatchItemTask($data);
			}
		}


		$data2['batch_status'] = 'Proses';
		$batchID = $batchModel->updateBatches($data2, $id);


		if ($batchID) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect' => ''
			];

			return json_encode($data);
		}
	}

	public function store_item_request($id){
		

		// bir_id bir_bi_id bir_bit_id	bir_item_sku bir_item_name bir_item_quantity	bir_item_weight

		$rules = [
            'item_sku' => ['label' => 'SKU', 'rules' => 'required|max_length[255]'],
            'item_name' => ['label' => 'Nama', 'rules' => 'required|max_length[255]'],
            'item_quantity' => ['label' => 'Jumlah', 'rules' => 'required|max_length[11]'],
            'item_weight' => ['label' => 'Weight', 'rules' => 'required|max_length[65]'],
        ];

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$data = [
			'bir_bi_id' => $id,
			'bir_bit_id' => $this->request->getPost('bit_id', FILTER_SANITIZE_STRING),
			'bir_item_sku' => $this->request->getPost('item_sku', FILTER_SANITIZE_STRING),
			'bir_item_name' => $this->request->getPost('item_name', FILTER_SANITIZE_STRING),
			'bir_item_quantity' => $this->request->getPost('item_quantity', FILTER_SANITIZE_STRING),
			'bir_item_weight' => $this->request->getPost('item_weight', FILTER_SANITIZE_STRING),
			'bir_insert_at' => getCurrentTime(),
			'bir_insert_admin_id' => $this->session->get('id'),
			'bir_active' => 1
		];

		$batchItemRequestModel = new BatchItemRequestModel();
		$result = $batchItemRequestModel->addBatchItemRequest($data);

		if ($result) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'reload' => true,
			];

			return json_encode($data);
		}
	}

	public function data(){
		// if (!$this->request->isAjax()) {
		// 	return false;
		// }

		$selectRepair = trim($this->request->getVar('select_repair', FILTER_SANITIZE_STRING));
		$fromDate = trim($this->request->getVar('from_date', FILTER_SANITIZE_STRING));
		$toDate = trim($this->request->getVar('to_date', FILTER_SANITIZE_STRING));
		$sku = trim($this->request->getVar('sku', FILTER_SANITIZE_STRING));
		$note = trim($this->request->getVar('note', FILTER_SANITIZE_STRING));
		$status = trim($this->request->getVar('status', FILTER_SANITIZE_STRING));

		$paramWhere = '';

		if ($selectRepair == 2) {
			$paramWhere .= ' AND (bit_2.bit_group_note IS NOT NULL OR bit_2.bit_group_note != "" )  ';
		}

		if ($selectRepair == 3) {
			$paramWhere .= ' AND (bit_2.bit_group_note IS NULL OR bit_2.bit_group_note = "" )  ';
		}

		if ($sku != '') {
			$paramWhere .= ' AND items.item_sku LIKE "%'.$sku.'%"  ';
		}

		if ($note != '') {
			$paramWhere .= ' AND (order_note LIKE "%'.$note.'%" OR batch_code LIKE "%'.$note.'%" OR bit_1.bit_group_note LIKE "%'.$note.'%")  ';
		}

		if ($fromDate != '') {
			$paramWhere .= ' AND DATE(bit_1.bit_insert_at) >= "'.$fromDate.'"  ';
		}

		if ($toDate != '') {
			$paramWhere .= ' AND DATE(bit_1.bit_insert_at) <= "'.$toDate.'"  ';
		}

		$paramGroupBy = '';

		if ($status == 'Completed') {
			$paramGroupBy = ' bit_1.bit_group_note HAVING is_completed=1 ';
		}else{
			$paramGroupBy = ' bit_1.bit_group_note';
			$paramWhere .= ' AND bit_1.bit_status IN ("Proses", "New") ';
		}

		$batchItemTaskModel = new BatchItemTaskModel();
		$batchItemTaskData = $batchItemTaskModel->getAllGroupNote(['where' => $paramWhere, 'groupby' => $paramGroupBy]);

		// dd($batchItemTaskData);

		// $dataRepaired = [];
		// if (@$batchItemTaskData) {
        // 	foreach ($batchItemTaskData as $index_ => $val_ ) {
        // 		$dataRepaired[] = $val_->repaired_bit_id;
		// 	}
    	// }

    	// $dataRepairedBit = []; 

    	// if (!empty($dataRepaired)) {
		// 	$batchItemTaskAll = $batchItemTaskModel->getAll(['where' => 'AND bit_id IN ('.implode(',', $dataRepaired).') ']);

		// 	foreach ($batchItemTaskAll as $key => $value) {
		// 		$dataRepairedBit[$value->id] = $value->group_note;
		// 	}

    	// }


		$data = [];

		if (@$batchItemTaskData) {
        	foreach ($batchItemTaskData as $index => $val ) {

        	$btn_print = '&nbsp;<a href="'.base_url().'/batch-item-task/print/'.$val->group_note.'" target="_blank"  class="btn btn-icon btn-warning" '.lang('Action.tooltip.print').'>'.lang('Action.icon.lg.print').'</a>';
        		
            $btn_task = '&nbsp;<a href="'.base_url().'/batch-item-task/'.$val->group_note.'" class="btn btn-primary btn-icon" '.lang('Action.tooltip.edit_task').' >'.lang('Action.icon.lg.edit_task').'</a>';

            $btn_request = '&nbsp;<a href="#" onclick="batchItemRequest('.$val->bi_id.')" class="btn btn-icon btn-success" data-bs-toggle="modal" data-bs-target="#kt_modal_1" '.lang('Action.tooltip.edit_request').' >'.lang('Action.icon.lg.edit_request').'</a>';

            $erpBarcode = '';
            if ($val->erp_barcode != '') {
            	$erpBarcode = '<small>ERP Barcode : '.$val->erp_barcode.'</small>';
            }

            $value['no'] = $index+1;
            $value['item_name'] = 'SKU : '.$val->item_sku.'<br>Nama : '.$val->item_name;
            $value['date'] = getTanggalWaktu($val->insert_at);
            $value['workflow_name'] = $val->workflow_name;
            $value['batch_code'] = $val->batch_code;
            // $value['tree_name'] = $val->tree_name;
            $value['group_note'] = $val->group_note.'<br>'.$erpBarcode;
            $value['task_name'] = $val->task_name;
            $value['repair_from'] = $val->group_note_2;
            $value['quantity'] = $val->input_quantity;
            $value['workflow_name'] = $val->workflow_name;
            $value['start_due_date'] = $val->start_due_date;
            $value['sku'] = $val->item_sku;
            $value['order_note'] = $val->order_note.'<br>Batch : '.$val->batch_code.'<br>Pohon : '.$val->tree_name;
            $value['action'] = $btn_task.$btn_print;
            
            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
	}

	public function edit($id){

		$batchItemTaskModel = new BatchItemTaskModel();
		$batchItemTaskData = $batchItemTaskModel->getAllDetail(['where' => ' AND batch_item_tasks.bit_group_note="'.$id.'" ', 'orderby' => ' batch_item_tasks.bit_id ASC ']);

		$requestTotal = $batchItemTaskModel->requestTotal($id);
		$meltTotal = $batchItemTaskModel->meltTotal($id);
		$outputTotal = $batchItemTaskModel->outputTotal($id);

		if($batchItemTaskData == false){
			echo "<script>window.close()</script>";
			exit();
		}

		if ($this->request->isAjax()) {
			$param = $this->request->getVar('param_type');
			if($param == 'repair'){
				return $this->repair($id);
			}else if($param == 'updateErpBarcode'){
				return $this->updateErpBarcode(@$batchItemTaskData[0]->bi_id);
			}else{
				return $this->update($id);
			}
		}


		$batchItemTaskCompleted = $batchItemTaskModel->getAllDetail(['where' => ' AND batch_item_tasks.bit_group_note="'.$id.'" AND (batch_item_tasks.bit_status="Completed" OR batch_item_tasks.bit_status="Proses" ) ', 'orderby' => ' batch_item_tasks.bit_id ASC ', 'groupby' => 'batch_item_tasks.bit_task_id']);

		$batchItemTaskProses = $batchItemTaskModel->getAllDetail(['where' => ' AND batch_item_tasks.bit_group_note="'.$id.'" AND batch_item_tasks.bit_status="Proses" ','orderby' => ' batch_item_tasks.bit_id ASC ']);

		// dd($batchItemTaskProses);

		$workflowModel = new WorkflowModel();

		$workflowData = $workflowModel->getWorkflowTaskGetByWorkflowRangeDown(@$batchItemTaskProses[0]->workflow_id, @$batchItemTaskProses[0]->order);
		
		$data = [
			'title' => lang('Menu.order.batch-item-task.main'),
			'menu' => 'batch-item-task',
			'data' => $batchItemTaskData,
			'data_completed' => $batchItemTaskCompleted,
			'data_task' => $workflowData,
			'batchItemID' => @$batchItemTaskData[0]->bi_id,
			'groupNote' => $id,
		];

		$batchItemTaskData = $batchItemTaskModel->getAllDetail(['where' => ' AND batch_item_tasks.bit_group_note="'.$id.'"  AND batch_item_tasks.bit_status="Proses" ', 'orderby' => ' batch_item_tasks.bit_id ASC ']);

		$bitID = @$batchItemTaskData[0]->id;

		$outgoingNoteDetailModel = new OutgoingNoteDetailModel();
		$outgoingNoteDetailData = $outgoingNoteDetailModel->getCustom([
			'select' => ' sum(ond_weight) as sum_total_weight, sum(ond_quantity) as "sum_total_quantity" ',
			'where' => ' AND on_types="Bahan Sisa" AND ond_bit_id="'.$bitID.'" AND on_active=1 ',
			'groupby' => '-'
		]);

		$data['sum_outgoing_note_weight'] = (isset($outgoingNoteDetailData[0]->sum_total_weight))? $outgoingNoteDetailData[0]->sum_total_weight: 0 ;
		$data['sum_outgoing_note_quantity'] = (isset($outgoingNoteDetailData[0]->sum_total_quantity))? $outgoingNoteDetailData[0]->sum_total_quantity: 0;

		$batchItemRequestModel = new BatchItemRequestModel();

		$batchItemRequestData = $batchItemRequestModel->getAll(['select' => ' SUM(bir_item_weight) as "sum_item_weight", SUM(bir_item_quantity) as "sum_item_quantity" ', 'where' => ' AND bir_bit_id="'.$bitID.'" AND bir_status IN ("New", "Approved") ', 'groupby' => '-']);

			
		$data['sum_request_item_weight'] = (isset($batchItemRequestData[0]->sum_item_weight))? $batchItemRequestData[0]->sum_item_weight:0;
		$data['sum_request_item_quantity'] = (isset($batchItemRequestData[0]->sum_item_quantity))? $batchItemRequestData[0]->sum_item_quantity : 0;


		$operatorTask = new OperatorTaskModel();
		$taskID =  @$batchItemTaskData[0]->task_id;
		$operatorData = $operatorTask->getAll(['where' => ' AND ot_task_id="'.$taskID.'" AND ot_active=1 ']);

		$data['operator_data'] = $operatorData;


		$requestTotalData = [];
		foreach ($requestTotal as $key => $value) {
			$requestTotalData[$value->bit_id] = $value;
		}

		$data['request_total_data'] = $requestTotalData;

		$meltTotalData = [];
		foreach ($meltTotal as $key => $value) {
			$meltTotalData[$value->bit_id] = $value;
		}

		$data['melt_total_data'] = $meltTotalData;

		$outputTotalData = [];
		foreach ($outputTotal as $key => $value) {
			$outputTotalData[$value->bit_id] = $value;
		}

		$data['output_total_data'] = $outputTotalData;

		return view('batch-item-task/edit', $data);
	}

	public function print($id){
		$batchItemTaskModel = new BatchItemTaskModel();
		$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" ', 'orderby' => ' bit_id ASC ']);

		$biID = @$batchItemTaskData[0]->bi_id;

		$batchItemRequestModel = new BatchItemRequestModel();
		$batchItemRequestData = $batchItemRequestModel->getAll(['where' => ' AND bir_bi_id="'.$biID.'" AND bir_active=1 ', 'orderby' => ' bir_id ASC ']);

		$batchItemModel = new BatchItemModel();
		$batchItemData = $batchItemModel->getByID($biID)[0];

		$barcode = new Barcode();
		$viewBarcode = $barcode->generate($id, "SVG");

		$batchItemOutputModel = new BatchItemOutputModel();

		$batchItemOutputData = $batchItemOutputModel->getAll(['where' => ' AND bio_bi_id="'.$biID.'" AND bio_active=1 ',  'orderby' => ' bio_id ASC ']);

		$data = [
			'title' => lang('Menu.order.batch-item-task.main'),
			'menu' => 'batch-item-task',
			'batch_item_task' => $batchItemTaskData,
			'batch_item' => $batchItemData,
			"batch_item_request" => $batchItemRequestData,
			'batch_item_output_data' => $batchItemOutputData, 
			'barcode' => $viewBarcode,
			'biID' => $biID
		];

		return view('batch-item-task/print', $data);
	}

	public function update($id){

		$bitID = $this->request->getPost('bit_id');
		$inputFields = $this->request->getPost('input_fields');
		$outputFields = $this->request->getPost('output_fields');
		$type = $this->request->getVar('type');

		$batchItemTaskModel = new BatchItemTaskModel();
		$bitData = $batchItemTaskModel->getAll(['where' => ' AND bit_id="'.$bitID.'" ', 'orderby' => ' bit_id ASC ']);

		$batchItemRequestModel = new BatchItemRequestModel();
		$batchItemRequestGet = $batchItemRequestModel->getAll(['where' => 'AND bir_item_source="Gudang Batu" AND bir_status="New" AND bir_bit_id="'.$bitID.'" AND bir_active=1 ']);

		if(count($batchItemRequestGet) > 0){
			$msg = [
                'error' => lang('ErrorMessages.materialNotYetApproved'),
            ];
            return json_encode($msg);
		}

		$taskOrder = $this->request->getVar("repair_task_order");

		$operatorModel = new OperatorModel();

		$rules = [];

		if ($type == 'input') {
			$dataInputField = json_decode($bitData[0]->input_fields);
			foreach ($dataInputField as $key => $value) {
				$textRules = '';
				if ($value->datatype == 'Integer' || $value->datatype == 'Decimal' ) {
					$textRules = 'trim|max_length[11]|numeric';
				}else{
					$textRules = 'trim|max_length[100]';
				}
				
				if ($value->is_required == 1) {
					$rules['input_fields.'.$bitID.'.'.$key]	= ['label' => $value->field_name, 'rules' => 'required|'.$textRules];
				}else{
					$rules['input_fields.'.$bitID.'.'.$key]	= ['label' => $value->field_name, 'rules' => $textRules];
				}
			}

		}else{
			$dataOutputField = json_decode($bitData[0]->output_fields);
			foreach ($dataOutputField as $key => $value) {
				$textRules = '';
				if ($value->datatype == 'Integer' || $value->datatype == 'Decimal' ) {
					$textRules = 'trim|max_length[11]|numeric';
				}else{
					$textRules = 'trim|max_length[100]';
				}
				
				if ($value->is_required == 1) {
					$rules['output_fields.'.$bitID.'.'.$key]	= ['label' => $value->field_name, 'rules' => 'required|'.$textRules];
				}else{
					$rules['output_fields.'.$bitID.'.'.$key]	= ['label' => $value->field_name, 'rules' => $textRules];
				}
			}
		}

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$outputItemQuantity = $this->request->getVar('output_item_quantity');
		$outputItemWeight = $this->request->getVar('output_item_weight');

		if ($outputItemQuantity != '' || $outputItemWeight != '') {
			$this->storeOutputMelt();
		}

		$data = [
            'bit_update_admin_id' => $this->session->get('id'),
            'bit_update_at' => getCurrentTime(),
        ];

		if ($type == 'input') {
			$resultInput = [];
			$dataInputField = json_decode($bitData[0]->input_fields);

			$quantity = 0;
			$weight = 0;
			$operator = '';
			$operatorID = null;
			$startAt = '';

			foreach ($dataInputField as $key => $value) {
				$value->answer = @$inputFields[$bitID][$key];

				if (@$value->options == 'quantity') {
					$quantity += intval(@$inputFields[$bitID][$key]);
				}

				if (@$value->options == 'weight') {
					$weight += @$inputFields[$bitID][$key];
				}

				if (@$value->options == 'operator') {
					$operatorID = @$inputFields[$bitID][$key];
					$operatorData = $operatorModel->getAll(['where' => 'AND operator_id="'.$operatorID.'" AND operator_active=1 ']);
					$operator = @$operatorData[0]->name;
					$value->answer = @$operator;

				}

				if (@$value->options == 'datetime') {
					$startAt = @$inputFields[$bitID][$key];
				}

				$resultInput[] = $value;
			}

			$data['bit_start_at'] = $startAt;
			$data['bit_input_fields'] = json_encode($resultInput);
            $data['bit_input_quantity']	= $quantity;
            $data['bit_input_weight'] = $weight;
            $data['bit_input_operator_name'] = $operator;
            $data['bit_input_operator_id'] = $operatorID;
            $data['bit_status'] = 'Proses';
            $data['bit_status_field'] = 'output';
		}else if ($type == 'output') {

			$resultOutput = [];
			$dataOutputField = json_decode($bitData[0]->output_fields);

			$quantity = 0;
			$weight = 0;
			$operator = '';
			$operatorID = null;
			$endAt = '';

			foreach ($dataOutputField as $key => $value) {
				$value->answer = @$outputFields[$bitID][$key];
				

				if (@$value->options == 'quantity') {
					$quantity += intval(@$outputFields[$bitID][$key]);
				}

				if (@$value->options == 'weight') {
					$weight += @$outputFields[$bitID][$key];
				}

				if (@$value->options == 'operator') {
					$operatorID = @$outputFields[$bitID][$key];
					$operatorData = $operatorModel->getAll(['where' => 'AND operator_id="'.$operatorID.'" AND operator_active=1 ']);
					$operator = @$operatorData[0]->name;
					$value->answer = @$operator;
				}

				if (@$value->options == 'datetime') {
					$endAt = @$outputFields[$bitID][$key];
				}

				$resultOutput[] = $value;
			}

			$data['bit_end_at'] = $endAt;
			$data['bit_output_fields'] = json_encode($resultOutput);
			$data['bit_output_quantity'] = $quantity;
            $data['bit_output_weight'] = $weight;
            $data['bit_output_operator_id'] = $operatorID;
            $data['bit_output_operator_name'] = $operator;
            $data['bit_status'] = 'Completed';
            $data['bit_status_field'] = '';
		}

		$result = $batchItemTaskModel->updateBatchItemTask($data, $bitID);


		$orderItemModel = new OrderItemModel();

		$repair = false;
		if($taskOrder != ''){
	    	$repair = $this->repair($id, $bitID);
		}

		if ($type == 'output') {

			$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" ', 'orderby' => ' bit_id ASC ']);

			$dataBit = [];
			foreach ($batchItemTaskData as $key => $value) {
				$dataBit[] = $value->id;
			}

			$keyBit = @array_keys($dataBit, $bitID)[0];

			if(isset($dataBit[$keyBit+1])){
				$bitID2 = $dataBit[$keyBit+1];

				$result2 = $batchItemTaskModel->updateBatchItemTask([
					'bit_update_admin_id' => $this->session->get('id'),
	            	'bit_update_at' => getCurrentTime(),
	            	'bit_status' => 'Proses',
	            	'bit_status_field' => 'input',
				], $bitID2);
			}else{
				$orderItemID = $bitData[0]->oi_id;

				$batchItemID = $bitData[0]->bi_id;
				$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_bi_id="'.$batchItemID.'" AND bit_status IN ("Proses", "New") ', 'orderby' => ' bit_id ASC ']);

				if (count($batchItemTaskData) > 0 ) {
					$dataOi = [
						'oi_status' => 'Proses',
			            'oi_update_admin_id' => $this->session->get('id'),
			            'oi_update_at' => getCurrentTime(),
			            'oi_active' => 1
					];
					$orderItemData = $orderItemModel->updateOrderItem($dataOi, $orderItemID);

				}else{

					$dataOi = [
						'oi_status' => 'Completed',
			            'oi_update_admin_id' => $this->session->get('id'),
			            'oi_completed_date' => date('Y-m-d'),
			            'oi_update_at' => getCurrentTime(),
			            'oi_active' => 1
					];
					$orderItemData = $orderItemModel->updateOrderItem($dataOi, $orderItemID);

				}
			}
		}else{

			$orderItemID = $bitData[0]->oi_id;

			$dataOi = [
				'oi_status' => 'Proses',
	            'oi_insert_admin_id' => $this->session->get('id'),
	            'oi_active'          => 1
			];
			
			$orderItemData = $orderItemModel->updateOrderItem($dataOi, $orderItemID);
		}


		$orderModel = new OrderModel();
		$orderData = $orderModel->getAll(['where' => ' AND order_active="1" AND order_id="'.@$bitData[0]->order_id.'" ']);
		if (isset($orderData)) {
			foreach ($orderData as $key => $value) {
				$orderItemData = $orderItemModel->getByOrderID($value->id);
				$orderItemDataCompleted = $orderItemModel->getByOrderIDStatus($value->id, 'Completed');

				$orderItemDataProses = $orderItemModel->getByOrderIDStatus($value->id, 'Proses');

				$dataOrder = [
		            'order_update_admin_id' => $this->session->get('id'),
		            'order_active'          => 1
		        ];

				if (count($orderItemData) == count($orderItemDataCompleted) ) {
					$dataOrder['order_status'] = 'Completed';
				}else if((count($orderItemData) > count($orderItemDataCompleted) &&  count($orderItemDataCompleted) > 0) || (count($orderItemData) >= count($orderItemDataProses) && count($orderItemDataProses) > 0)  ){
					$dataOrder['order_status'] = 'Proses';
				}else{
					$dataOrder['order_status'] = 'New';
				}

				$orderData = $orderModel->updateOrder($dataOrder, $value->id); 
			}
		}


		// $bitID = $this->request->getVar('bit_id');


		$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" AND bit_status IN ("Proses", "New") ', 'orderby' => ' bit_id ASC ']);

		
		if (count($batchItemTaskData) == 0) {

			$batchItemOutputModel = new BatchItemOutputModel();

			$batchItemOutputData = $batchItemOutputModel->getAll(['where' => 'AND bio_bit_id ="'.$bitID.'" AND bio_active=1 AND bio_is_melt=0 ']);

			if(!$batchItemOutputData){
				$data = [
					'bio_bi_id'=> @$bitData[0]->bi_id,
					'bio_oi_id'=> @$bitData[0]->oi_id,
					'bio_bit_id' => $bitID,
					'bio_item_id'=> @$bitData[0]->item_id,
					'bio_is_melt'=> 0,
					'bio_note'=> $this->bioNote(date('Y-m-d')),
					'bio_item_sku' => @$bitData[0]->item_sku,
					'bio_item_name' => @$bitData[0]->item_name,
					'bio_item_quantity' =>$quantity,
					'bio_item_weight' => $weight,
					'bio_insert_at' => getCurrentTime(),
					'bio_insert_admin_id' => $this->session->get('id')
				];
			
				$result = $batchItemOutputModel->addBatchItemOutput($data);
			}

		
				$outputJson = [
					'success'=> lang('Validation.success_add'),
					'redirect' => 'batch-item-task'
				];

				$batchItemID = $bitData[0]->bi_id;
				$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_bi_id="'.$batchItemID.'" AND bit_status IN ("Proses", "New") ', 'orderby' => ' bit_id ASC ']);


				if (count($batchItemTaskData) == 0 ) {

					$batchItemOutputData = $batchItemOutputModel->getAll(['select' => ' SUM(bio_item_quantity) as sum_item_quantity, SUM(bio_item_weight) as sum_item_weight ', 'where' => ' AND bio_bi_id='.$batchItemID.' AND bio_is_melt=0 AND bio_active=1  ', 'orderby' => ' bio_id ASC ']);

					$completedQuantity = @$batchItemOutputData[0]->sum_item_quantity;
					$completedWeight = @$batchItemOutputData[0]->sum_item_weight;

					$dataOi = [
						'oi_status' => 'Completed',
			            'oi_update_admin_id' => $this->session->get('id'),
			            'oi_completed_date' => date('Y-m-d'),
			            'oi_completed_quantity' => $completedQuantity,
			            'oi_completed_weight' => $completedWeight,
			            'oi_update_at' => getCurrentTime(),
			            'oi_active' => 1
					];
					
					$orderItemData = $orderItemModel->updateOrderItem($dataOi, @$bitData[0]->oi_id);

					$batchItemModel = new BatchItemModel();
					$dataBi = [
						'bi_completed_quantity' => $completedQuantity,
						'bi_completed_weight' => $completedWeight,
			            'bi_update_admin_id' => $this->session->get('id'),
			            'bi_update_at' => getCurrentTime(),
			            'bi_active' => 1
					];
					$batchItemData = $batchItemModel->updateBatchItem($dataBi, @$bitData[0]->bi_id);

					$batchItemGroupNoteModel = new BatchItemGroupNoteModel();
					$dataBign = [
						'bign_bi_id' => @$bitData[0]->bi_id,
						'bign_bit_group_note' => $id,
						'bign_completed_quantity' => $completedQuantity,
						'bign_completed_weight' => $completedWeight,
						'bign_on_id' => NULL,
						'bign_status'=> 'New',
						'bign_insert_at' => getCurrentTime(), 
						'bign_insert_admin_id' => $this->session->get('id'),
						'bign_update_at' => getCurrentTime(),
						'bign_update_admin_id' => $this->session->get('id'),
					];
					$batchItemGroupNoteData = $batchItemGroupNoteModel->addBatchItemGroupNote($dataBign);

				}

		}else{
			$outputJson = [
				'success'=> lang('Validation.success_add'),
				'reload' => true
			];
		}

		$batchItemTaskData2 = $batchItemTaskModel->getAll(['where' => ' AND batches.batch_id="'.$bitData[0]->batch_id.'" AND bit_status="Proses" ', 'orderby' => ' bit_id ASC ']);

		if (empty($batchItemTaskData2)) {

			$batchModel = new BatchModel();

			$data = [
				'batch_status' 		=> 'Completed',
				'batch_update_admin_id' => $this->session->get('id'),
				'batch_update_at' 		=> getCurrentTime(),
	            'batch_active'          => 1
	        ];

			$batchID = $batchModel->updateBatches($data, $bitData[0]->batch_id);

			$outputJson = [
				'success'=> lang('Validation.success_add'),
				'redirect' => 'batch-item-task'
			];

		}

		return json_encode($outputJson);
	
	}


	public function report(){
		if($this->request->isAjax()){
            return $this->dataReport();
        }

        if ($this->request->getGet('export-excel') == true) {
            return $this->exportReport();
        }

        $data = [
			'title' => lang('Menu.report.batch-item-task.main'),
			'menu' => 'batch-item-task-report',
		];

		return view('batch-item-task/report', $data);
	}

	public function dataReport(){
		if (!$this->request->isAjax()) {
			return false;
		}

		$note = $this->request->getVar('note', FILTER_SANITIZE_STRING);
		$batchCode = $this->request->getVar('batch_code', FILTER_SANITIZE_STRING);
		$sku = $this->request->getVar('sku', FILTER_SANITIZE_STRING);
		$fromDateNote = $this->request->getVar('from_date_note', FILTER_SANITIZE_STRING);
 		$toDateNote = $this->request->getVar('to_date_note', FILTER_SANITIZE_STRING);
	 	$fromDateStart = $this->request->getVar('from_date_start', FILTER_SANITIZE_STRING);
		$toDateStart = $this->request->getVar('to_date_start', FILTER_SANITIZE_STRING);
		$operator = $this->request->getVar('operator', FILTER_SANITIZE_STRING);
		$status = $this->request->getVar('status', FILTER_SANITIZE_STRING);
		$statusNote = $this->request->getVar('status_note', FILTER_SANITIZE_STRING);

        $queryWhere = ' AND bit_repaired_bit_id IS NULL ';

        if ($fromDateNote != '' && $toDateNote != '') {
            $queryWhere .= ' AND DATE(order_date) >="'.$fromDateNote.'" ';
            $queryWhere .= ' AND DATE(order_date) <= "'.$toDateNote.'" ';
        }

        if ($fromDateStart != '' && $toDateStart != '') {
            $queryWhere .= ' AND DATE(batch_item_tasks.bit_start_at) >="'.trim($fromDateStart).'" ';
            $queryWhere .= ' AND DATE(batch_item_tasks.bit_start_at) <= "'.trim($toDateStart).'" ';
        }

        if ($fromDateNote == '' && $toDateNote == '' && $fromDateStart == '' && $toDateStart == ''){
            $queryWhere .= ' AND MONTH(order_date)="'.date('m').'" AND YEAR(order_date)="'.date('Y').'" ';
        }

        if ($sku != '') {
            $queryWhere .= ' AND item_sku = "'.trim($sku).'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND (batch_code = "'.trim($batchCode).'" OR bit_group_note = "'.trim($batchCode).'") ';
        }

        if ($note != '') {
            $queryWhere .= ' AND order_note = "'.trim($note).'" ';
        }

        if ($status != '') {
            $queryWhere .= ' AND bit_status = "'.trim($status).'" ';
        }

        if( $statusNote != '' ){
        	$queryWhere .= ' AND order_status = "'.trim($statusNote).'" ';
        }

        if ($operator != '') {
            $queryWhere .= ' AND bit_input_operator_name LIKE "%'.trim($operator).'%" OR bit_output_operator_name LIKE "%'.trim($operator).'%" ';
        }
		
		$batchItemTaskModel = new BatchItemTaskModel();

		$batchItemTaskData = $batchItemTaskModel->getAll(['where'=> $queryWhere, 'orderby' => ' order_note ASC, batch_code ASC, bit_insert_at ASC,  bit_start_at ASC ']);

		$data = [];

		if (@$batchItemTaskData) {
        	foreach ($batchItemTaskData as $index => $val ) {

	            $value['note'] = $val->order_note;
	            $value['batch_code'] = $val->batch_code;
	            $value['group_note'] = $val->group_note;
	            $value['gold_content'] = $val->gold_content;
	            $value['item_sku'] = $val->item_sku;
	            $value['item_name'] = $val->item_name;
	            $value['task_name'] = $val->task_name;
	            $value['status'] = $val->status;
	            $value['input_quantity'] = $val->input_quantity;
	            $value['output_quantity'] = $val->output_quantity;
	            $value['input_weight'] = $val->input_weight;
	            $value['output_weight'] = $val->output_weight;
	            $value['start_date'] = $val->start_at.'<br>'.$val->input_operator_name;

	            if ($val->status == 'Completed') {
	            	$value['end_date'] = $val->update_at.'<br>'.$val->output_operator_name;
	            }else{
	            	$value['end_date'] = '';
	            }

	            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
	}

	public function exportReport(){
		$note = $this->request->getVar('note', FILTER_SANITIZE_STRING);
		$batchCode = $this->request->getVar('batch_code', FILTER_SANITIZE_STRING);
		$sku = $this->request->getVar('sku', FILTER_SANITIZE_STRING);
		$fromDateNote = $this->request->getVar('from_date_note', FILTER_SANITIZE_STRING);
 		$toDateNote = $this->request->getVar('to_date_note', FILTER_SANITIZE_STRING);
	 	$fromDateStart = $this->request->getVar('from_date_start', FILTER_SANITIZE_STRING);
		$toDateStart = $this->request->getVar('to_date_start', FILTER_SANITIZE_STRING);

		$operator = $this->request->getVar('operator', FILTER_SANITIZE_STRING);
        $status = $this->request->getVar('status', FILTER_SANITIZE_STRING);
		

        $queryWhere = ' AND bit_repaired_bit_id IS NULL ';

        if ($fromDateNote != '' && $toDateNote != '') {
            $queryWhere .= ' AND DATE(order_date) >="'.$fromDateNote.'" ';
            $queryWhere .= ' AND DATE(order_date) <= "'.$toDateNote.'" ';
        }

        if ($fromDateStart != '' && $toDateStart != '') {
            $queryWhere .= ' AND DATE(batch_item_tasks.bit_start_at) >="'.trim($fromDateStart).'" ';
            $queryWhere .= ' AND DATE(batch_item_tasks.bit_start_at) <= "'.trim($toDateStart).'" ';
        }

        if ($fromDateNote == '' && $toDateNote == '' && $fromDateStart == '' && $toDateStart == ''){
            $queryWhere .= ' AND MONTH(order_date)="'.date('m').'" AND YEAR(order_date)="'.date('Y').'" ';
        }

        if ($sku != '') {
            $queryWhere .= ' AND item_sku = "'.$sku.'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND (batch_code = "'.$batchCode.'" OR bit_group_note = "'.$batchCode.'") ';
        }

        if ($note != '') {
            $queryWhere .= ' AND order_note = "'.$note.'" ';
        }

		if ($status != '') {
            $queryWhere .= ' AND bit_status = "'.trim($status).'" ';
        }

        if ($operator != '') {
            $queryWhere .= ' AND bit_input_operator_name LIKE "%'.trim($operator).'%" OR bit_output_operator_name LIKE "%'.trim($operator).'%" ';
        }
		
		$batchItemTaskModel = new BatchItemTaskModel();

		$batchItemTaskData = $batchItemTaskModel->getAll(['where'=> $queryWhere, 'orderby' => ' order_note ASC, batch_code ASC, bit_insert_at ASC,  bit_start_at ASC ']);

		$data = [
			'title' => lang('Menu.order.batch-item-task.main'),
			'menu' => 'batch-item-task',
			'data' => $batchItemTaskData,
		];

		return view('batch-item-task/excel', $data);
	}

	public function repair($id, $bitID){

		$batchItemTaskModel = new BatchItemTaskModel();
		$taskModel = new TaskModel();

		$taskOrder = $this->request->getVar("repair_task_order");
		$weight = $this->request->getVar("repair_weight");
		$quantity = $this->request->getVar("repair_quantity");

		$rules = [];

		foreach ($taskOrder as $key => $value) {
	        $rules['repair_task_order.'.$key] = ['label' => 'Repair Order Task', 'rules' => 'required|max_length[11]'];
	        $rules['repair_weight.'.$key] = ['label' => 'Repair Berat', 'rules' => 'required|max_length[11]'];
	        $rules['repair_quantity.'.$key] = ['label' => 'Repair Jumlah', 'rules' => 'required|max_length[11]'];
		}


		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$workflowModel = new WorkflowModel();

		foreach ($taskOrder as $keyTask => $valueTask) {

			$batchItemTaskProses = $batchItemTaskModel->getAll(['where' => ' AND bit_id="'.$bitID.'" ','orderby' => ' bit_id ASC ']);
			$bitID = @$batchItemTaskProses[0]->id;
			$biID = @$batchItemTaskProses[0]->bi_id;
			$workflowID = $batchItemTaskProses[0]->workflow_id;

			$workflowTaskData = $workflowModel->getWorkflowTaskGetByWorkflowRangeUp($workflowID, $valueTask);

			$batchItemTaskCompleted = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" AND (bit_status="Completed" ) ', 'orderby' => ' bit_id ASC ']);
			
			$groupNote = $this->groupNote();

			$data_ = [];
			foreach ($batchItemTaskCompleted as $key_ => $value_) {
				$data_[] = [
					'bit_bi_id' => $value_->bi_id,
					'bit_order' => $value_->order,
					'bit_is_repaired' => $value_->is_repaired,
					'bit_repaired_bit_id' => $bitID,
					'bit_workflow_id' => $value_->workflow_id,
					'bit_task_id' => $value_->task_id,
					'bit_group_note' => $groupNote,
					'bit_input_operator_name' => $value_->input_operator_name,
					'bit_output_operator_name' => $value_->output_operator_name,
					'bit_input_fields' => $value_->input_fields,
					'bit_output_fields' => $value_->output_fields,
					'bit_input_quantity' => $value_->input_quantity,
					'bit_output_quantity' => $value_->output_quantity,
					'bit_input_weight' => $value_->input_weight,
					'bit_output_weight' => $value_->output_weight,
					'bit_start_at' => $value_->start_at,
					'bit_end_at' => $value_->end_at,
					'bit_output_weight' => $value_->output_weight,
					'bit_status_field' => $value_->status_field,
					'bit_status' => 'Completed',
					'bit_insert_at' => $value_->insert_at, 
					'bit_insert_admin_id' => $value_->insert_admin_id
				];
			}

			$result = $batchItemTaskModel->addBatchItemTaskBulk($data_);


			$data = [];
			foreach ($workflowTaskData as $key => $value) {

				$taskInputData = [];
				$taskResult = $taskModel->taskInputFieldGetByTask($value->task_id, 'input');
				foreach ($taskResult as $key2 => $value2) {
					$taskInputData[$key2]['field_name'] = $value2->name;
					$taskInputData[$key2]['is_required'] = $value2->is_required;
					$taskInputData[$key2]['datatype'] = $value2->datatype;
					$taskInputData[$key2]['options'] = $value2->options;
					$taskInputData[$key2]['answer'] = '';
					if($key == 0){
						if($value2->options == 'quantity'){
							$taskInputData[$key2]['answer'] = @$quantity[$keyTask];
						}else if($value2->options == 'weight'){
							$taskInputData[$key2]['answer'] = @$weight[$keyTask];
						}
					}
				}

				$taskOutputData = [];
				$taskResult3 = $taskModel->taskInputFieldGetByTask($value->task_id, 'output');
				foreach ($taskResult3 as $key3 => $value3) {
					$taskOutputData[$key3]['field_name'] = $value3->name;
					$taskOutputData[$key3]['is_required'] = $value3->is_required;
					$taskOutputData[$key3]['datatype'] = $value3->datatype;
					$taskOutputData[$key3]['options'] = $value3->options;
					$taskOutputData[$key3]['answer'] = '';
					// if($key == 0){
					// 	if($value2->options == 'quantity'){
					// 		$taskInputData[$key3]['answer'] = @$quantity[$keyTask];
					// 	}else if($value2->options == 'weight'){
					// 		$taskInputData[$key3]['answer'] = @$weight[$keyTask];
					// 	}
					// }
				}

				$data = [
					'bit_bi_id' => $biID,
					'bit_order' => $value->order,
					'bit_is_repaired' => ($key == 0)? 1 : 0,
					'bit_repaired_bit_id' => NULL,
					'bit_workflow_id' => $value->workflow_id,
					'bit_task_id' => $value->task_id,
					'bit_group_note' => $groupNote,
					'bit_input_operator_name' => '',
					'bit_output_operator_name' => '',
					'bit_input_fields' => json_encode($taskInputData),
					'bit_output_fields' => json_encode($taskOutputData),
					'bit_input_quantity' => ($key == 0)? @$quantity[$keyTask] : 0,
					'bit_output_quantity' => 0,
					'bit_input_weight' => ($key == 0)? @$weight[$keyTask] : 0,
					'bit_output_weight' => 0,
					'bit_start_at' => NULL,
					'bit_end_at' => NULL,
					'bit_status_field' => ($key == 0)? 'input' : '',
					'bit_status' => ($key == 0)? 'Proses' : 'New',
					'bit_insert_at' => getCurrentTime(),
					'bit_insert_admin_id' => $this->session->get('id')
				];

				$result = $batchItemTaskModel->addBatchItemTask($data);
			}


			if ($result) {
				if ($keyTask == count($taskOrder)) {
					return true;
				}
			}
		}

		return false;

	}

	public function groupNote(){
		$batchItemTaskModel = new BatchItemTaskModel();
		$bitNota = $batchItemTaskModel->setNote();

		if ($bitNota > 0) {
			$number = $bitNota;
	        $number = ((int)$number) + 1;
	        $code = str_pad($number, 5, "0", STR_PAD_LEFT);
		}else{
			$code = '00001';
		}

		$dateEx = explode('-', date('Y-m-d'));

		return 'BIT'.$dateEx[2].$dateEx[1].substr($dateEx[0], -2) .''.$code;
	}


	public function storeOutputMelt(){

		$rules = [
            'output_item_quantity' => ['label' => 'Jumlah', 'rules' => 'required|max_length[11]'],
            'output_item_weight' => ['label' => 'Weight', 'rules' => 'required|max_length[65]'],
        ];

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$biID = $this->request->getVar('output_bi_id');

		$bitID = $this->request->getVar('bit_id');

		$batchModel = new BatchModel();
		$batchItemOutputModel = new BatchItemOutputModel();

		$batchItemModel = new BatchItemModel();

		$biData = $batchItemModel->getByID($biID);

		if ($biData == false) {
			$msg = [
                'error' => lang('Validation.data_not_found'),
            ];
            return json_encode($msg);
		}
		
		$data = [
			'bio_bi_id'=> $biID,
			'bio_oi_id'=> @$biData[0]->order_item_id,
			'bio_item_id'=> @$biData[0]->item_id,
			'bio_bit_id' => $bitID,
			'bio_is_melt'=> 1,
			'bio_note'=> $this->bioNote(date('Y-m-d')),
			'bio_item_sku' => @$biData[0]->sku,
			'bio_item_name' => @$biData[0]->name,
			'bio_item_quantity' => $this->request->getVar('output_item_quantity', FILTER_SANITIZE_STRING),
			'bio_item_weight' => $this->request->getVar('output_item_weight', FILTER_SANITIZE_STRING),
			'bio_insert_at' => getCurrentTime(),
			'bio_insert_admin_id' => $this->session->get('id')
		];

		$result = $batchItemOutputModel->addBatchItemOutput($data);

		if ($result) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'modal_close' => '#kt_modal_melt'
			];

			return json_encode($data);
		}
	}

	
	public function bioNote($date){

		$batchItemOutputModel = new BatchItemOutputModel();
		$bitNota = $batchItemOutputModel->setNote();

		if ($bitNota) {
			$number = $bitNota[0]->total;
	        $number = ((int)$number) + 1;
	        $code = str_pad($number, 5, "0", STR_PAD_LEFT);
		}else{
			$code = '00001';
		}

		$dateEx = explode('-', $date);

		return 'ONBP/'.$dateEx[2].$dateEx[1].substr($dateEx[0], -2) .'/'.$code;
	}

	public function updateErpBarcode($biID){

		$erpBarcode = $this->request->getPost('erp_barcode');

		$rules = [
            'erp_barcode' => ['label' => 'ERP Barcode', 'rules' => 'required|max_length[256]'],
        ];

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

    	$dataBi = [
			'bi_erp_barcode' => @$erpBarcode,
            'bi_update_admin_id' => $this->session->get('id'),
            'bi_update_at' => $this->session->get('id'),
            'bi_active'    => 1
		];

		$batchItemModel = new BatchItemModel();

		$batchItemData = $batchItemModel->updateBatchItem($dataBi, $biID);

		if ($batchItemData) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'reload' => true
			];

			return json_encode($data);
		}else{

			$data = [
				'error'=> lang('Validation.error_add'),
			];

			return json_encode($data);
		}

    }


    public function editTask($id){

    	if($this->request->isAjax()){
			return $this->updateTask($id);
		}

    	$batchItemTaskModel = new BatchItemTaskModel();
		$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_id="'.$id.'" ', 'orderby' => ' bit_id ASC ']);

		$data = [
            'title' => 'Edit '. lang('Menu.order.batch-item-task.main'),
            'data' => $batchItemTaskData[0],
            'menu' => 'batch-item-task',
        ];

        return view('batch-item-task/edit-task', $data);

    }

    public function updateTask($id){

		$inputFields = $this->request->getPost('input_fields');
		$outputFields = $this->request->getPost('output_fields');
		$type = $this->request->getVar('type');

		$batchItemOutputModel = new BatchItemOutputModel();

		$batchItemOutputData = $batchItemOutputModel->getAll(['where' => ' AND bio_bit_id="'.$id.'" AND bio_active=1 AND bio_on_id IS NOT NULL AND bio_is_melt != 1']);
		if(count($batchItemOutputData) > 0){
			$msg = [
                'error' => lang('ErrorMessages.alreadyOutgoingBatchCantBeEdited'),
            ];
            return json_encode($msg);
		}

		$batchItemTaskModel = new BatchItemTaskModel();
		$bitData = $batchItemTaskModel->getAll(['where' => ' AND bit_id="'.$id.'" ', 'orderby' => ' bit_id ASC ']);
		
		$rowBitData = $bitData[0];

		$dataLog = [
			'bitl_bit_id' => $id,
			'bitl_workflow_id' => $rowBitData->workflow_id,
			'bitl_task_id' => $rowBitData->task_id,
			'bitl_group_note' => $rowBitData->group_note,
			'bitl_old_input_fields' => $rowBitData->input_fields, 
			'bitl_old_output_fields' => $rowBitData->output_fields, 
			'bitl_update_at' => getCurrentTime(),
			'bitl_update_admin_id' => $this->session->get('id')
		];

		$rules = [];

		$dataInputField = json_decode($bitData[0]->input_fields);
		foreach ($dataInputField as $key => $value) {
			$textRules = '';
			if ($value->datatype == 'Integer' || $value->datatype == 'Decimal' ) {
				$textRules = 'trim|max_length[11]|numeric';
			}else{
				$textRules = 'trim|max_length[100]';
			}
			
			if ($value->is_required == 1) {
				$rules['input_fields.'.$key]	= ['label' => $value->field_name, 'rules' => 'required|'.$textRules];
			}else{
				$rules['input_fields.'.$key]	= ['label' => $value->field_name, 'rules' => $textRules];
			}
		}

		$dataOutputField = json_decode($bitData[0]->output_fields);
		foreach ($dataOutputField as $key => $value) {
			$textRules = '';
			if ($value->datatype == 'Integer' || $value->datatype == 'Decimal' ) {
				$textRules = 'trim|max_length[11]|numeric';
			}else{
				$textRules = 'trim|max_length[100]';
			}
			
			if ($value->is_required == 1) {
				$rules['output_fields.'.$key]	= ['label' => $value->field_name, 'rules' => 'required|'.$textRules];
			}else{
				$rules['output_fields.'.$key]	= ['label' => $value->field_name, 'rules' => $textRules];
			}
		}
		

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$outputItemQuantity = $this->request->getVar('output_item_quantity');
		$outputItemWeight = $this->request->getVar('output_item_weight');

		if ($outputItemQuantity != '' || $outputItemWeight != '') {
			$this->storeOutputMelt();
		}

		$data = [
            'bit_update_admin_id' => $this->session->get('id'),
            'bit_update_at' => getCurrentTime(),
        ];

		$resultInput = [];
		$dataInputField = json_decode($bitData[0]->input_fields);

		$quantity = 0;
		$weight = 0;
		$operator = '';
		$startAt = '';

		foreach ($dataInputField as $key => $value) {
			$value->answer = @$inputFields[$key];
			$resultInput[] = $value;

			if (@$value->options == 'quantity') {
				$quantity += intval(@$inputFields[$key]);
			}

			if (@$value->options == 'weight') {
				$weight += @$inputFields[$key];
			}

			if (@$value->options == 'operator') {
				$operator = @$inputFields[$key];
			}

			if (@$value->options == 'datetime') {
				$startAt = @$inputFields[$key];
			}
		}

		$data['bit_start_at'] = $startAt;
		$data['bit_input_fields'] = json_encode($resultInput);
        $data['bit_input_quantity']	= $quantity;
        $data['bit_input_weight'] = $weight;
        $data['bit_input_operator_name'] = $operator;


		$resultOutput = [];
		$dataOutputField = json_decode($bitData[0]->output_fields);

		$quantity = 0;
		$weight = 0;
		$operator = '';
		$endAt = '';

		foreach ($dataOutputField as $key => $value) {
			$value->answer = @$outputFields[$key];
			$resultOutput[] = $value;

			if (@$value->options == 'quantity') {
				$quantity += intval(@$outputFields[$key]);
			}

			if (@$value->options == 'weight') {
				$weight += @$outputFields[$key];
			}

			if (@$value->options == 'operator') {
				$operator = @$outputFields[$key];
			}

			if (@$value->options == 'datetime') {
				$endAt = @$outputFields[$key];
			}
		}

		$data['bit_end_at'] = $endAt;
		$data['bit_output_fields'] = json_encode($resultOutput);
		$data['bit_output_quantity'] = $quantity;
        $data['bit_output_weight'] = $weight;
        $data['bit_output_operator_name'] = $operator;

		$result = $batchItemTaskModel->updateBatchItemTask($data, $id);

		$dataLog['bitl_new_input_fields'] = json_encode($resultInput);
		$dataLog['bitl_new_output_fields'] = json_encode($resultOutput);

		$batchItemTaskLogModel = new BatchItemTaskLogModel();

		$batchItemOutputData = $batchItemOutputModel->getAll(['where' => ' AND bio_bit_id="'.$id.'" AND bio_active=1 AND bio_on_id IS NULL AND bio_is_melt != 1 ']);
		if($batchItemOutputData){

			$data = [
				'bio_item_quantity' => $quantity,
				'bio_item_weight' => $weight,
				'bio_update_at' => getCurrentTime(),
				'bio_update_admin_id' => $this->session->get('id')
			];
			$result = $batchItemOutputModel->updateBatchItemOutput($data, $batchItemOutputData[0]->id);
		}

		$addLog = $batchItemTaskLogModel->addBatchItemTaskLog($dataLog);

		if ($result) {
			
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'batch-item-task/'.$rowBitData->group_note
			];

			return json_encode($data);
		}

    }

}
